<html>
<head>

    <title>django-mongonaut</title>
    <link rel="stylesheet" type="text/css" href="{{ MONGONAUT_TWITTER_BOOTSTRAP }}">
    <script src="{{ MONGONAUT_JQUERY }}" type="text/javascript"></script>
    <script src="{{ MONGONAUT_TWITTER_BOOTSTRAP_ALERT }}" type="text/javascript"></script>

</head>


<body>
<ul class="breadcrumb">
    {% block breadcrumbs %}
    {% endblock %}
</ul>
<div class="container-fluid">

    {% if messages %}
    <div class="alert-messages">
        {% for message in messages %}
        <div class="alert {% if message.tags %} alert-{{ message.tags }} {% endif %}">
            <a class="close" data-dismiss="alert">&times;</a>
            <p>{{ message }}</p>
        </div>
        {% endfor %}
    </div>
    {% endif %}

{% block content %}{% endblock %}
    <div class="row">
        <div class="span12">
            <footer>
                <p>
                    &copy; 2012 <a href="//pydanny.com/category/blog.html">Daniel Greenfeld</a>
                    - <a href="//github.com/pydanny/django-mongonaut">Fork on Github!</a>
                    - <a href="//django-mongonaut.rtfd.org">Documentation</a>
                </p>
            </footer>
        </div>
    </div>
</div>

{% block extrajs %}
{% endblock %}

<script type="text/javascript">
//<![CDATA[
{% block inlinejs %}
{% endblock %}
//]]>
</script>

<script type="text/javascript">
    $("#action_delete").click(function(){
        $('.btn-group').hide();
        $('#delete_warning').show();
    });
    $(".alert_close").click(function(){
        $('.btn-group').show();
        $('#delete_warning').hide();
    });
    $("#action_delete_ok").click(function(){
        var form_ = $('#delete_form');
        form_.submit();
    });
</script>

</body>
</html>
