{% extends "mongonaut/base.html" %}
{% load url from future %}
{% load mongonaut_tags %}

{% block breadcrumbs %}
<li><a href="{% url "index" %}">Mongonaut</a> <span class="divider">/</span></li>
<li><a href="{% url "document_list" app_label document_name %}">{{ document_name }}s</a> <span class="divider">/</span></li>
<li><a href="{% url "document_detail" app_label document_name document.id %}">{{ document_name }} {{ document.id }}</a> <span class="divider">/</span></li>
<li><a href="">Edit</a>
{% endblock %}

{% block content %}

<h1>Edit {{ document_name }}</h1>

{% include "mongonaut/includes/form.html" %}

{% endblock %}

{% block extrajs %}
<form method="post" id="delete_form" action="{% url "document_delete" app_label document_name document.id %}">
{% csrf_token %}
</form>
{% endblock %}

{% block inlinejs %}
{% include 'mongonaut/includes/list_add.js' %}
{% endblock %}

